######################################################
#' @title Figure 1 (aggregation methods) Generation
#' @author A. Grant Schissler
#' @date 15 Jan 2016
#' @description This script generates the basic visualizations and numeric summaries in Figure 1
#' @note All figures/tables were subsequently edited in Adobe Illustrator (Ai)
######################################################

### Load Libraries
#install.packages("ggplot2")
require(ggplot2)

####################################
##### Create Figure 1, Panel A
####################################

###################
## Create Parameters
## Set Parameters
pathway = "go.bp"
name <- "TCGA.38.4625"

## change directory
setwd("./Figure1")

### Read in example data
### retrieve the first pathway
patient.data <- read.csv(paste(name,".csv",sep = ""),row.names=1)

## Load Gene Set Database to use
pathways.set = read.csv(file=paste("./",pathway,".filtered.csv",sep=""))

## compute vertical mah dist for each pathway
## Functions for Mahalanobis Distance
compute.vert.dist <- function(normal,tumor){
    sqrt((var(normal)/((var(normal)*var(tumor))-cov(normal,tumor)^2)))*(tumor-normal)
}

## split the pathways into a list
symbols.list <- split(pathways.set$symbol,pathways.set$path_id)
tmp <- symbols.list[[1]]
goTerm <- names(symbols.list)[1]

## create figure for one pathway
sym <- as.character(tmp)
tmp.data <- patient.data[row.names(patient.data) %in% sym,]
tmp.data <- tmp.data[order(tmp.data$Normal),]

## lab color scheme
## purple "#A078AA"
## orange "#FFA040"
## green "#72BF44"
p <- ggplot(tmp.data,aes(x=Normal,y=Tumor))
p <- p + geom_abline(slope=1,color="#A078AA") + geom_point(color="#FFA040") + theme_bw() + labs(title=paste("Paired Gene Expression within",goTerm), x="Log2 Gene Expression for Normal Tissue", y="Log2 Gene Expression for Tumor")
p <- p + geom_text(data=tmp.data[c(1,7,nrow(tmp.data)),],family="mono",fontface=2,size=4,label=paste("Gene",c(1,"j","m")),vjust=1.5,hjust=0.496)
p <- p + geom_segment(aes(x=tmp.data[7,1],y=tmp.data[7,2],xend=tmp.data[7,1],yend=tmp.data[7,1]),color="#72BF44")
ggsave(plot=p,height=6,width=6,dpi=200, filename="Figure1.Panel.A.pdf", useDingbats=FALSE)


####################################
##### Create Figure 1, Panel B
####################################

## setwd("./Figure1")
### Plot density of median MD scores for cross group
## load pre-computed bootstrap distributions
load("../CommonData/CTC_tumor_resample.RData") ## all patients

## pull first pathway to create density
md.data <- data.frame(MD = boot.null.med[,1])
(obs.med <- abs(median(md.matrix[b.a.index,1], na.rm = T)))
p0 <- ggplot(md.data, aes(MD))
(p1 <- p0 + geom_density(fill = "white", color = "black", adjust = 3))
(p2 <- p1 + geom_vline(xintercept = obs.med, col = "red") + geom_vline(xintercept = -obs.med, col = "red"))
(p3 <- p2 + theme_bw() + labs(y = "", x = ""))
## save
ggsave("Figure1.Panel.B.pdf")

####################################
##### Create Figure 1, Panel C
####################################

## setwd("./Figure1")
load("../CommonData/CTC_pairs.RData")

### lab colors:
purple <- "#A078AA"
orange <- "#FFA040"
green <- "#72BF44"
grey <- "#777676"
yellow <- "#F9F080"

################################
### 1. Format Data and Compute metrics

## from Dropbox/CTCs/Figures/Figure_Dysreg_Group/Figure_Dysreg_Group.R
#### retrieve percent dysregulated
## create teriary indicator for pathway
## tmp.data <- data.list[[5]]
threshold <- 0.05
sig.indicator.list <- lapply(data.list, function(tmp.data) {
    tmp.data <- tmp.data[pid.id,]
    tmp.result <- as.numeric(tmp.data$p.value < threshold & !is.nan(tmp.data$distance))
    ## replace NaN MD distance with 0
    tmp.data$distance[is.nan(tmp.data$distance)] <- 0
    tmp.result*sign(tmp.data$distance)
})

sig.indicator.matrix <- do.call("rbind",sig.indicator.list)
colnames(sig.indicator.matrix) <- pid.id

#### restrict to the top five pathways
## top pathway IDs
top.ids <- c("200020", "200141", "200135", "200142", "200085")
top.labels <- c("ncWnt","ErbB2/B3","SDC4", "FOXM1", "S1P1")
top.ind <- sig.indicator.matrix[,top.ids]
## uncomment this to summarize all pathways
## top.ind <- sig.indicator.matrix

### create data frame for plotting 
top.ind.data <- data.frame(top.ind)

## label remaining pairings
top.ind.data$Pairing <- "EMPTY"
top.ind.data[a.a.index,"Pairing"] <- "N"
top.ind.data[b.b.index,"Pairing"] <- "R"
top.ind.data[b.a.index,"Pairing"] <- "B-A"

## remove B-A comparisons
top.ind.data <- top.ind.data[-b.a.index,]

## remove within patient comparisons
patient.pairs <- unlist(lapply(strsplit(rownames(top.ind.data), "[\\.|-]"), function(tmp) tmp[1] == tmp[4]))
top.ind.data <- top.ind.data[!patient.pairs,]

######################################################
### 2. Plot distribution of MD scores

### summarize to write in long format to construct bar plot
## note: technically a weighted cross-patient, within group observed proportion
df <- split(top.ind.data, top.ind.data$Pairing)[[1]] ## Naive patients in ncWnt
table(factor(df[,1]))/length(df)
md.data <- data.frame(DEP = factor(df[,1], labels = c("Down", "Non-Sig", "Up")))
## create plot
p0 <- ggplot(md.data, aes(x = DEP))
p1 <- p0 + geom_bar(aes(fill = DEP))
p2 <- p1 + scale_fill_manual(values = c("#993404", "gray", "#08519C"))
(p3 <- p2 + theme_bw() + labs(y = "", x = ""))
## save plot
ggsave("Figure1.Panel.C.pdf")

## note that up and down are arbitray and so were combined in the final figure

####################################
##### Create Figure 1, Panel D
####################################
## load data and patient information from CTC Pairs
load("../CommonData/CTC_pairs.RData")

### lab colors:
purple <- "#A078AA"
orange <- "#FFA040"
green <- "#72BF44"
grey <- "#777676"
yellow <- "#F9F080"

######################################################
### 1. Format Data and Compute metrics

### first retrieve the actual MD Score
top.md.data <- data.frame(md.matrix)
names(top.md.data) <- gsub("^X","",names(top.md.data))

#### restrict to just B-A pairs
top.md.data <- top.md.data[b.a.index,]

#### restrict to the top five pathways
## top pathway IDs
top.ids <- c("200020", "200141", "200135", "200142", "200085")
top.labels <- c("ncWnt","ErbB2/B3","SDC4", "FOXM1", "S1P1")
## subset to only top five
top.md.data <- top.md.data[,top.ids]

## check for missing MD scores
any(is.na(top.md.data))

### now retrieve the siginifance pathways
## count genesets under a certain FDR
type <- c("BY","p.value","Bon","Holm")
tmp.type <- type[2]


## choose one CTC and pathway to display MD distribution 
tmp.ctc <- "Pr21.1.10"
tmp.id <- "200141"

## resistant patients have exactly 1 median CTC pair
tmp.md <- top.md.data[ grep(paste0(tmp.ctc,"[-|$]"), rownames(top.md.data)), tmp.id]

## find the median pair
tmp.index <- which(tmp.md == median(tmp.md, na.rm = T))
tmp.pair <- rownames(top.md.data)[grep(paste0(tmp.ctc,"[-|$]"), rownames(top.md.data))][tmp.index]

## lookup pvalue
tmp.pvalue <- data.list[[tmp.pair]][tmp.id,tmp.type]

## lookup MD score
tmp.dist <- data.list[[tmp.pair]][tmp.id,"distance"]

######################################################
### 2. Plot distribution of MD scores
md.data <- data.frame(MD = tmp.md)

p0 <- ggplot(md.data, aes(MD))
p1 <- p0 + geom_histogram(binwidth = 0.5, fill = "white", color = "black")
p2 <- p1 + geom_vline(xintercept = tmp.dist, col = "red")
(p3 <- p2 + theme_bw() + labs(y = "", x = ""))

ggsave("Figure1.Panel.D.pdf")

